;;;;
;;;; Gtk-Server Example
;;;; By Leonardo Cecchi
;;;;

(defpackage :example-five
  (:use :common-lisp :gtk-server)
  (:export :main))
(in-package :example-five)

(defclass input-entry (gui-component)
  ((text-entry)))

(defcomponent input-entry (label-desc)
  (let ((my-entry (make-instance 'input-entry)))
    (with-components
	((box (hbox 0 2))
	 (label (label label-desc))
	 (entry (entry)))
      (with-pack-in box
	((:padding 2 :fill 0 :expand 0) label)
	((:padding 2 :fill 1 :expand 1) entry))
      (setf (slot-value my-entry 'text-entry) entry)
      (setf (widget my-entry) box)
      my-entry)))

(defgeneric get-text (self))
(defmethod get-text ((self input-entry))
  "Get text of an input entry"
  (gtk_entry_get_text (slot-value self 'text-entry)))

(defun main ()
  "Simple custom component example"
  (with-gtk-server '()
    (with-components
	((win (window 0))
	 (box (vbox 1 2))
	 (ie-name (input-entry "Name:"))
	 (ie-surname (input-entry "Surname:"))
	 (bt-ok (button "Ok")))

      ;; Pack and show
      (with-pack-in box
	((:padding 2 :fill 1 :expand 1) ie-surname)
	((:padding 2 :fill 1 :expand 1) ie-name)
	((:padding 2 :fill 0 :expand 0) bt-ok))
      (gtk_window_set_title win "Custom Component Example")
      (gtk_container_add win box)
      (gtk_widget_show_all win)

      ;; Events
      (with-event-loop my-event
	(win
	 (return))
	(bt-ok
	 (let ((nome (get-text ie-name))
	       (cognome (get-text ie-surname)))
	   (cond
	     ((= 0 (length nome))
	      (show-error-dialog win "Insert name!"))
	     ((= 0 (length cognome))
	      (show-error-dialog win "Insert surname!"))
	     (t
	      (show-info-dialog win (format nil "Welcome ~a ~a" nome cognome))
	      (return)))))))))
