;;;;
;;;; My Stupid and simple text editor
;;;; By Leonardo Cecchi
;;;;

(defpackage :example-seven
  (:use :common-lisp :gtk-server)
  (:export :main))
(in-package :example-seven)

;;; My custom components
(defcomponent menuitem-with-child (label &rest childs)
  "Simple components who make a menu from childs and add
this menu to che item create with label"
  (with-components ((itm-menu (menuitem-l label))
		    (submenu (menu)))
    (loop for child in childs
	  do (gtk_menu_shell_append submenu child))
    (gtk_menu_item_set_submenu itm-menu submenu)
    itm-menu))

(defcomponent menubar-with-child (&rest childs)
  "Simple menu bar with menu item childs"
  (with-components ((bar (menubar)))
    (loop for child in childs
	  do (gtk_menu_shell_append bar child))
    bar))

;;; Text editor component

(defclass textedit-component (gui-component)
  ((f-textview :accessor f-textview)
   (box :accessor box)
   (filename :accessor filename)))

(defgeneric textexit-open-file (te parent)
  (:documentation "File open action"))
(defgeneric textedit-new-file (te parent)
  (:documentation "File new action"))
(defgeneric textedit-save-as-file (te parent)
  (:documentation "File 'save as' action"))
(defgeneric textedit-save-file (te parent)
  (:documentation "File save action"))
(defgeneric textedit-window-title (te)
  (:documentation "Return window title"))

(defcomponent textedit ()
  "Really simple text editor component who incapsulates
a textview component"
  (let ((instance (make-instance 'textedit-component)))
    (with-components ((tv (textview))
		      (vbox (vbox 0 2))
		      (scroll (scrolledwindow)))
      (setf (f-textview instance) tv)
      (setf (filename instance) nil)
      (gtk_container_add scroll tv)
      (setf (box instance) vbox)
      (with-pack-in vbox
	((:padding 2 :fill 1 :expand 1) scroll))
      (setf (widget instance) vbox)
      instance)))

(defmethod textedit-new-file ((te textedit-component) parent)
  "Delete all test from textview and set the file names
to nil"
  (block controllo
    (when (filename te)
      (unless (show-yes-no-dialog parent "You are currently editing a file.
If you open an other file you will lose the changes.
Would you like to continue?")
	(return-from controllo)))
    (gtk_text_view_set_text (f-textview te) "")
    (setf (filename te) nil)))

(defmethod textedit-open-file ((te textedit-component) parent)
  "Open a dialog box with an open file request and if
the user say ok load the file"
  (block controllo
    (let ((file-chooser (gtk_file_chooser_dialog_new "Choose a file to open..."
						     parent
						     gtk-server::+GTK_FILE_CHOOSER_ACTION_OPEN+
						     "Ok" 1
						     "Cancel" 2)))
      ;; Close and exit if cancel
      (unless (= 1 (gtk_dialog_run file-chooser))
	(gtk_widget_destroy file-chooser)
	(return-from controllo))
    
      (let ((filename (gtk_file_chooser_get_filename file-chooser)))
	;; Close and exit if the file doesn't exists
	(unless (probe-file filename)
	  (return-from controllo))
	;; Read the file
	(with-open-file (in filename :direction :input)
	  (let ((buffer (format nil "~{~a~%~}"
				(loop for line = (read-line in nil)
				      until (null line)
				      collecting line))))
	    (gtk_text_view_set_text (f-textview te) buffer)))
	(setf (filename te) filename))
      (gtk_widget_destroy file-chooser))))

(defmethod textedit-save-as-file ((te textedit-component) parent)
  "Save the file in the component with a new filename"
  (block controllo
    (let ((file-chooser (gtk_file_chooser_dialog_new "Choose a file name..."
						     parent
						     gtk-server::+GTK_FILE_CHOOSER_ACTION_SAVE+
						     "Ok" 1
						     "Cancel" 2)))
      ;; Close and exit if the user don't want to save the file
      (unless (= 1 (gtk_dialog_run file-chooser))
	(gtk_widget_destroy file-chooser)
	(return-from controllo))
      
      ;; Want to save the file?? Ok.....
      (let ((filename (filename te))
	    (filename-chooser (gtk_file_chooser_get_filename file-chooser)))
	(when (and (not (equal filename filename-chooser)) (probe-file filename-chooser))
	  (unless (show-yes-no-dialog parent "You are saving on an existing file.
The original file will be overridden.
Would you like to continue?")
	    (gtk_widget_destroy file-chooser)
	    (return-from controllo)))
	(setf (filename te) filename-chooser)
	(textedit-save-file te parent)
	(gtk_widget_destroy file-chooser)))))

(defmethod textedit-save-file ((te textedit-component) parent)
  "Save the file in the component with the filename in the instance"
  (block check
    (unless (filename te)
      (show-error-dialog parent "You have to choose the 'save as'
action because the current buffer doesn't have
a filename.")
      (return-from check))
    
    (with-open-file (out (filename te) :direction :output
			 :if-exists :supersede)
      (format out "~a~%" (gtk_text_view_get_text (f-textview te))))))

(defmethod textedit-window-title ((te textedit-component))
  "Return the preferred window title"
  (cond
    ((null (filename te)) "My really simple text editor")
    (t (format nil "My really simple text editor. Editing: '~a'" (filename te)))))

;;; My really stupid text editor
(defun main ()
  (with-gtk-server '()
      (with-components ((win (window 0))
			;; File Menu
			(itm-new (menuitem-l "New file"))
			(itm-open (menuitem-l "Open File..."))
			(itm-sep1 (menuitem))
			(itm-save (menuitem-l "Save"))
			(itm-save-as (menuitem-l "Save As..."))
			(itm-sep2 (menuitem))
			(itm-exit (menuitem-l "Exit"))
			(itm-file (menuitem-with-child "File"
						       itm-new
						       itm-open
						       itm-sep1
						       itm-save
						       itm-save-as
						       itm-sep2
						       itm-exit))
			;; Edit Menu
			(itm-copy (menuitem-l "Copy (only example)"))
			(itm-cut (menuitem-l "Cut (only example)"))
			(itm-paste (menuitem-l "Paste (only example)"))
			(itm-edit (menuitem-with-child "Edit"
						       itm-copy
						       itm-cut
						       itm-paste))
			;; Menu Bar
			(bar (menubar-with-child itm-file itm-edit))
			;; Text Editing component
			(textedit (textedit))
			;; Boxes
			(vbox (vbox 0 2)))
	;; Other window customizations
	(with-pack-in vbox
	  ((:padding 0 :fill 0 :expand 0) bar)
	  ((:padding 0 :fill 1 :expand 1) textedit))
	(gtk_window_set_title win "My really simple text editor")
	(gtk_container_add win vbox)
	(gtk_window_set_default_size win 500 500)
	(gtk_widget_show_all win)

	;; Event loop
	(with-event-loop my-event
	  (win
	   (return))
	  (itm-exit
	   (return))
	  (itm-open
	   (textedit-open-file textedit win)
	   (gtk_window_set_title win (textedit-window-title textedit)))
	  (itm-new
	   (textedit-new-file textedit win)
	   (gtk_window_set_title win (textedit-window-title textedit)))
	  (itm-save
	   (textedit-save-file textedit win)
	   (gtk_window_set_title win (textedit-window-title textedit)))
	  (itm-save-as
	   (textedit-save-as-file textedit win)
	   (gtk_window_set_title win (textedit-window-title textedit)))))))
